"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "utils", {
  enumerable: true,
  get: function get() {
    return _utils["default"];
  }
});
exports["default"] = exports.getErrorDescription = exports.ErrorCodes = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _hwTransport = require("@ledgerhq/hw-transport");

var _utils = _interopRequireWildcard(require("./utils"));

var _ErrorMsgs;

function _createForOfIteratorHelper(o) { if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) { var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var it, normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(n); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var CLA = 0xd7;
var INS = {
  GET_VERSION: 0x00,
  GET_EXT_PUBLIC_KEY: 0x10,
  DERIVE_ADDRESS: 0x11,
  ATTEST_UTXO: 0x20,
  SIGN_TX: 0x21,
  RUN_TESTS: 0xf0
};
var ErrorCodes = {
  ERR_STILL_IN_CALL: 0x6e04,
  // internal
  ERR_INVALID_DATA: 0x6e07,
  ERR_INVALID_BIP_PATH: 0x6e08,
  ERR_REJECTED_BY_USER: 0x6e09,
  ERR_REJECTED_BY_POLICY: 0x6e10,
  ERR_DEVICE_LOCKED: 0x6e11,
  // Not thrown by ledger-app-cardano itself but other apps
  ERR_CLA_NOT_SUPPORTED: 0x6e00
};
exports.ErrorCodes = ErrorCodes;
var GH_ERRORS_LINK = "https://github.com/cardano-foundation/ledger-app-cardano/blob/master/src/errors.h";
var ErrorMsgs = (_ErrorMsgs = {}, (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_INVALID_DATA, "Invalid data supplied to Ledger"), (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_INVALID_BIP_PATH, "Invalid derivation path supplied to Ledger"), (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_REJECTED_BY_USER, "Action rejected by user"), (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_REJECTED_BY_POLICY, "Action rejected by Ledger's security policy"), (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_DEVICE_LOCKED, "Device is locked"), (0, _defineProperty2["default"])(_ErrorMsgs, ErrorCodes.ERR_CLA_NOT_SUPPORTED, "Wrong Ledger app"), _ErrorMsgs);

var getErrorDescription = function getErrorDescription(statusCode) {
  var statusCodeHex = "0x".concat(statusCode.toString(16));
  var defaultMsg = "General error ".concat(statusCodeHex, ". Please consult ").concat(GH_ERRORS_LINK);
  return ErrorMsgs[statusCode] || defaultMsg;
}; // It can happen that we try to send a message to the device
// when the device thinks it is still in a middle of previous ADPU stream.
// This happens mostly if host does abort communication for some reason
// leaving ledger mid-call.
// In this case Ledger will respond by ERR_STILL_IN_CALL *and* resetting its state to
// default. We can therefore transparently retry the request.
// Note though that only the *first* request in an multi-APDU exchange should be retried.


exports.getErrorDescription = getErrorDescription;

var wrapRetryStillInCall = function wrapRetryStillInCall(fn) {
  return /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
    var _args = arguments;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return fn.apply(void 0, _args);

          case 3:
            return _context.abrupt("return", _context.sent);

          case 6:
            _context.prev = 6;
            _context.t0 = _context["catch"](0);

            if (!(_context.t0 && _context.t0.statusCode && _context.t0.statusCode === ErrorCodes.ERR_STILL_IN_CALL)) {
              _context.next = 12;
              break;
            }

            _context.next = 11;
            return fn.apply(void 0, _args);

          case 11:
            return _context.abrupt("return", _context.sent);

          case 12:
            throw _context.t0;

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 6]]);
  }));
};

var wrapConvertError = function wrapConvertError(fn) {
  return /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
    var _args2 = arguments;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            _context2.next = 3;
            return fn.apply(void 0, _args2);

          case 3:
            return _context2.abrupt("return", _context2.sent);

          case 6:
            _context2.prev = 6;
            _context2.t0 = _context2["catch"](0);

            if (_context2.t0 && _context2.t0.statusCode) {
              // keep HwTransport.TransportStatusError
              // just override the message
              _context2.t0.message = "Ledger device: ".concat(getErrorDescription(_context2.t0.statusCode));
            }

            throw _context2.t0;

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[0, 6]]);
  }));
};
/**
 * Cardano ADA API
 *
 * @example
 * import Ada from "@ledgerhq/hw-app-ada";
 * const ada = new Ada(transport);
 */


var Ada = /*#__PURE__*/function () {
  function Ada(transport) {
    var scrambleKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "ADA";
    (0, _classCallCheck2["default"])(this, Ada);
    this.transport = transport;
    this.methods = ["getVersion", "getExtendedPublicKey", "signTransaction", "deriveAddress", "showAddress"];
    this.transport.decorateAppAPIMethods(this, this.methods, scrambleKey);
    this.send = wrapConvertError(this.transport.send);
  }
  /**
   * Returns an object containing the app version.
   *
   * @returns {Promise<GetVersionResponse>} Result object containing the application version number.
   *
   * @example
   * const { major, minor, patch, flags } = await ada.getVersion();
   * console.log(`App version ${major}.${minor}.${patch}`);
   *
   */


  (0, _createClass2["default"])(Ada, [{
    key: "getVersion",
    value: function () {
      var _getVersion = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
        var _this = this;

        var _send, P1_UNUSED, P2_UNUSED, response, _response, major, minor, patch, flags_value, FLAG_IS_DEBUG, flags;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _send = function _send(p1, p2, data) {
                  return _this.send(CLA, INS.GET_VERSION, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                P1_UNUSED = 0x00;
                P2_UNUSED = 0x00;
                _context3.next = 5;
                return wrapRetryStillInCall(_send)(P1_UNUSED, P2_UNUSED, _utils["default"].hex_to_buf(""));

              case 5:
                response = _context3.sent;

                _utils.Assert.assert(response.length == 4);

                _response = (0, _slicedToArray2["default"])(response, 4), major = _response[0], minor = _response[1], patch = _response[2], flags_value = _response[3];
                FLAG_IS_DEBUG = 1; //const FLAG_IS_HEADLESS = 2;

                flags = {
                  isDebug: (flags_value & FLAG_IS_DEBUG) == FLAG_IS_DEBUG
                };
                return _context3.abrupt("return", {
                  major: major,
                  minor: minor,
                  patch: patch,
                  flags: flags
                });

              case 11:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      function getVersion() {
        return _getVersion.apply(this, arguments);
      }

      return getVersion;
    }()
    /**
     * Runs unit tests on the device (DEVEL app build only)
     *
     * @returns {Promise<void>}
     */

  }, {
    key: "runTests",
    value: function () {
      var _runTests = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4() {
        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return wrapRetryStillInCall(this.send)(CLA, INS.RUN_TESTS, 0x00, 0x00);

              case 2:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function runTests() {
        return _runTests.apply(this, arguments);
      }

      return runTests;
    }()
  }, {
    key: "_attestUtxo",
    value: function () {
      var _attestUtxo2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(txDataHex, outputIndex) {
        var _this2 = this;

        var _send, P1_INIT, P1_CONTINUE, P2_UNUSED, CHUNK_SIZE, data, result, txData, i, chunk, _result, _chunk, _result2, sum, sizes, _utils$chunkBy, _utils$chunkBy2, txHash, outputNumber, amount, hmac;

        return _regenerator["default"].wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _utils.Precondition.checkIsHexString(txDataHex);

                _utils.Precondition.checkIsUint32(outputIndex);

                _send = function _send(p1, p2, data) {
                  return _this2.send(CLA, INS.ATTEST_UTXO, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                P1_INIT = 0x01;
                P1_CONTINUE = 0x02;
                P2_UNUSED = 0x00; // Note: U2F transport does not like APDU length longer than 255
                // so we are leaving it some space

                CHUNK_SIZE = 245;
                // Initial request
                data = _utils["default"].uint32_to_buf(outputIndex);
                _context5.next = 10;
                return wrapRetryStillInCall(_send)(P1_INIT, P2_UNUSED, data);

              case 10:
                result = _context5.sent;

                _utils.Assert.assert(result.length == 0);

                txData = _utils["default"].hex_to_buf(txDataHex);
                i = 0;

              case 14:
                if (!(i + CHUNK_SIZE < txData.length)) {
                  _context5.next = 23;
                  break;
                }

                chunk = txData.slice(i, i + CHUNK_SIZE);
                _context5.next = 18;
                return _send(P1_CONTINUE, P2_UNUSED, chunk);

              case 18:
                _result = _context5.sent;

                _utils.Assert.assert(_result.length == 0);

                i += CHUNK_SIZE;
                _context5.next = 14;
                break;

              case 23:
                _chunk = txData.slice(i);
                _context5.next = 26;
                return _send(P1_CONTINUE, P2_UNUSED, _chunk);

              case 26:
                _result2 = _context5.sent;

                sum = function sum(arr) {
                  return arr.reduce(function (x, y) {
                    return x + y;
                  }, 0);
                };

                sizes = [32, 4, 8, 16];

                _utils.Assert.assert(_result2.length == sum(sizes));

                _utils$chunkBy = _utils["default"].chunkBy(_result2, sizes), _utils$chunkBy2 = (0, _slicedToArray2["default"])(_utils$chunkBy, 4), txHash = _utils$chunkBy2[0], outputNumber = _utils$chunkBy2[1], amount = _utils$chunkBy2[2], hmac = _utils$chunkBy2[3];
                return _context5.abrupt("return", {
                  rawBuffer: _result2,
                  txHashHex: _utils["default"].buf_to_hex(txHash),
                  outputIndex: _utils["default"].buf_to_uint32(outputNumber),
                  amountStr: _utils["default"].buf_to_amount(amount),
                  hmacHex: _utils["default"].buf_to_hex(hmac)
                });

              case 32:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5);
      }));

      function _attestUtxo(_x, _x2) {
        return _attestUtxo2.apply(this, arguments);
      }

      return _attestUtxo;
    }()
    /**
     * @param string Raw transaction data (without witnesses) encoded as hex string
     * @param number Output indes
     *
     */

  }, {
    key: "attestUtxo",
    value: function () {
      var _attestUtxo3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(txDataHex, outputIndex) {
        return _regenerator["default"].wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                return _context6.abrupt("return", this._attestUtxo(txDataHex, outputIndex));

              case 1:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function attestUtxo(_x3, _x4) {
        return _attestUtxo3.apply(this, arguments);
      }

      return attestUtxo;
    }()
    /**
     * @description Get a public key from the specified BIP 32 path.
     *
     * @param {BIP32Path} indexes The path indexes. Path must begin with `44'/1815'/n'`, and may be up to 10 indexes long.
     * @return {Promise<GetExtendedPublicKeyResponse>} The public key with chaincode for the given path.
     *
     * @example
     * const { publicKey, chainCode } = await ada.getExtendedPublicKey([ HARDENED + 44, HARDENED + 1815, HARDENED + 1 ]);
     * console.log(publicKey);
     *
     */

  }, {
    key: "getExtendedPublicKey",
    value: function () {
      var _getExtendedPublicKey = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(path) {
        var _this3 = this;

        var _send, P1_UNUSED, P2_UNUSED, data, response, _utils$chunkBy3, _utils$chunkBy4, publicKey, chainCode, rest;

        return _regenerator["default"].wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _utils.Precondition.checkIsValidPath(path);

                _send = function _send(p1, p2, data) {
                  return _this3.send(CLA, INS.GET_EXT_PUBLIC_KEY, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                P1_UNUSED = 0x00;
                P2_UNUSED = 0x00;
                data = _utils["default"].path_to_buf(path);
                _context7.next = 7;
                return wrapRetryStillInCall(_send)(P1_UNUSED, P2_UNUSED, data);

              case 7:
                response = _context7.sent;
                _utils$chunkBy3 = _utils["default"].chunkBy(response, [32, 32]), _utils$chunkBy4 = (0, _slicedToArray2["default"])(_utils$chunkBy3, 3), publicKey = _utils$chunkBy4[0], chainCode = _utils$chunkBy4[1], rest = _utils$chunkBy4[2];

                _utils.Assert.assert(rest.length == 0);

                return _context7.abrupt("return", {
                  publicKeyHex: publicKey.toString("hex"),
                  chainCodeHex: chainCode.toString("hex")
                });

              case 11:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7);
      }));

      function getExtendedPublicKey(_x5) {
        return _getExtendedPublicKey.apply(this, arguments);
      }

      return getExtendedPublicKey;
    }()
    /**
     * @description Gets an address from the specified BIP 32 path.
     *
     * @param {BIP32Path} indexes The path indexes. Path must begin with `44'/1815'/i'/(0 or 1)/j`, and may be up to 10 indexes long.
     * @return {Promise<DeriveAddressResponse>} The address for the given path.
     *
     * @throws 5001 - The path provided does not have the first 3 indexes hardened or 4th index is not 0 or 1
     * @throws 5002 - The path provided is less than 5 indexes
     * @throws 5003 - Some of the indexes is not a number
     *
     * @example
     * const { address } = await ada.deriveAddress([ HARDENED + 44, HARDENED + 1815, HARDENED + 1, 0, 5 ]);
     *
     */

  }, {
    key: "deriveAddress",
    value: function () {
      var _deriveAddress = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(path) {
        var _this4 = this;

        var _send, P1_RETURN, P2_UNUSED, data, response;

        return _regenerator["default"].wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _utils.Precondition.checkIsValidPath(path);

                _send = function _send(p1, p2, data) {
                  return _this4.send(CLA, INS.DERIVE_ADDRESS, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                P1_RETURN = 0x01;
                P2_UNUSED = 0x00;
                data = _utils["default"].path_to_buf(path);
                _context8.next = 7;
                return _send(P1_RETURN, P2_UNUSED, data);

              case 7:
                response = _context8.sent;
                return _context8.abrupt("return", {
                  address58: _utils["default"].base58_encode(response)
                });

              case 9:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8);
      }));

      function deriveAddress(_x6) {
        return _deriveAddress.apply(this, arguments);
      }

      return deriveAddress;
    }()
  }, {
    key: "showAddress",
    value: function () {
      var _showAddress = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(path) {
        var _this5 = this;

        var _send, P1_DISPLAY, P2_UNUSED, data, response;

        return _regenerator["default"].wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _utils.Precondition.checkIsValidPath(path);

                _send = function _send(p1, p2, data) {
                  return _this5.send(CLA, INS.DERIVE_ADDRESS, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                P1_DISPLAY = 0x02;
                P2_UNUSED = 0x00;
                data = _utils["default"].path_to_buf(path);
                _context9.next = 7;
                return _send(P1_DISPLAY, P2_UNUSED, data);

              case 7:
                response = _context9.sent;

                _utils.Assert.assert(response.length == 0);

              case 9:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9);
      }));

      function showAddress(_x7) {
        return _showAddress.apply(this, arguments);
      }

      return showAddress;
    }()
  }, {
    key: "signTransaction",
    value: function () {
      var _signTransaction = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee16(inputs, outputs) {
        var _this6 = this;

        var P1_STAGE_INIT, P1_STAGE_INPUTS, P1_STAGE_OUTPUTS, P1_STAGE_CONFIRM, P1_STAGE_WITNESSES, P2_UNUSED, SIGN_TX_INPUT_TYPE_ATTESTED_UTXO, _send, signTx_init, signTx_addInput, signTx_addAddressOutput, signTx_addChangeOutput, signTx_awaitConfirm, signTx_getWitness, attestedInputs, _iterator, _step, _step$value, txDataHex, outputIndex, _attestation, _i, _attestedInputs, attestation, _iterator2, _step2, output, _yield$signTx_awaitCo, txHashHex, witnesses, _iterator3, _step3, input, witness;

        return _regenerator["default"].wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                //console.log("sign");
                P1_STAGE_INIT = 0x01;
                P1_STAGE_INPUTS = 0x02;
                P1_STAGE_OUTPUTS = 0x03;
                P1_STAGE_CONFIRM = 0x04;
                P1_STAGE_WITNESSES = 0x05;
                P2_UNUSED = 0x00;
                SIGN_TX_INPUT_TYPE_ATTESTED_UTXO = 0x01;

                _send = function _send(p1, p2, data) {
                  return _this6.send(CLA, INS.SIGN_TX, p1, p2, data).then(_utils["default"].stripRetcodeFromResponse);
                };

                signTx_init = /*#__PURE__*/function () {
                  var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10(numInputs, numOutputs) {
                    var data, response;
                    return _regenerator["default"].wrap(function _callee10$(_context10) {
                      while (1) {
                        switch (_context10.prev = _context10.next) {
                          case 0:
                            data = Buffer.concat([_utils["default"].uint32_to_buf(numInputs), _utils["default"].uint32_to_buf(numOutputs)]);
                            _context10.next = 3;
                            return wrapRetryStillInCall(_send)(P1_STAGE_INIT, P2_UNUSED, data);

                          case 3:
                            response = _context10.sent;

                            _utils.Assert.assert(response.length == 0);

                          case 5:
                          case "end":
                            return _context10.stop();
                        }
                      }
                    }, _callee10);
                  }));

                  return function signTx_init(_x10, _x11) {
                    return _ref3.apply(this, arguments);
                  };
                }();

                signTx_addInput = /*#__PURE__*/function () {
                  var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(attestation) {
                    var data, response;
                    return _regenerator["default"].wrap(function _callee11$(_context11) {
                      while (1) {
                        switch (_context11.prev = _context11.next) {
                          case 0:
                            data = Buffer.concat([_utils["default"].uint8_to_buf(SIGN_TX_INPUT_TYPE_ATTESTED_UTXO), attestation.rawBuffer]);
                            _context11.next = 3;
                            return _send(P1_STAGE_INPUTS, P2_UNUSED, data);

                          case 3:
                            response = _context11.sent;

                            _utils.Assert.assert(response.length == 0);

                          case 5:
                          case "end":
                            return _context11.stop();
                        }
                      }
                    }, _callee11);
                  }));

                  return function signTx_addInput(_x12) {
                    return _ref4.apply(this, arguments);
                  };
                }();

                signTx_addAddressOutput = /*#__PURE__*/function () {
                  var _ref5 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee12(address58, amountStr) {
                    var data, response;
                    return _regenerator["default"].wrap(function _callee12$(_context12) {
                      while (1) {
                        switch (_context12.prev = _context12.next) {
                          case 0:
                            data = Buffer.concat([_utils["default"].amount_to_buf(amountStr), _utils["default"].uint8_to_buf(0x01), _utils["default"].base58_decode(address58)]);
                            _context12.next = 3;
                            return _send(P1_STAGE_OUTPUTS, P2_UNUSED, data);

                          case 3:
                            response = _context12.sent;

                            _utils.Assert.assert(response.length == 0);

                          case 5:
                          case "end":
                            return _context12.stop();
                        }
                      }
                    }, _callee12);
                  }));

                  return function signTx_addAddressOutput(_x13, _x14) {
                    return _ref5.apply(this, arguments);
                  };
                }();

                signTx_addChangeOutput = /*#__PURE__*/function () {
                  var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee13(path, amountStr) {
                    var data, response;
                    return _regenerator["default"].wrap(function _callee13$(_context13) {
                      while (1) {
                        switch (_context13.prev = _context13.next) {
                          case 0:
                            data = Buffer.concat([_utils["default"].amount_to_buf(amountStr), _utils["default"].uint8_to_buf(0x02), _utils["default"].path_to_buf(path)]);
                            _context13.next = 3;
                            return _send(P1_STAGE_OUTPUTS, P2_UNUSED, data);

                          case 3:
                            response = _context13.sent;

                            _utils.Assert.assert(response.length == 0);

                          case 5:
                          case "end":
                            return _context13.stop();
                        }
                      }
                    }, _callee13);
                  }));

                  return function signTx_addChangeOutput(_x15, _x16) {
                    return _ref6.apply(this, arguments);
                  };
                }();

                signTx_awaitConfirm = /*#__PURE__*/function () {
                  var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee14() {
                    var response;
                    return _regenerator["default"].wrap(function _callee14$(_context14) {
                      while (1) {
                        switch (_context14.prev = _context14.next) {
                          case 0:
                            _context14.next = 2;
                            return _send(P1_STAGE_CONFIRM, P2_UNUSED, _utils["default"].hex_to_buf(""));

                          case 2:
                            response = _context14.sent;
                            return _context14.abrupt("return", {
                              txHashHex: response.toString("hex")
                            });

                          case 4:
                          case "end":
                            return _context14.stop();
                        }
                      }
                    }, _callee14);
                  }));

                  return function signTx_awaitConfirm() {
                    return _ref7.apply(this, arguments);
                  };
                }();

                signTx_getWitness = /*#__PURE__*/function () {
                  var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee15(path) {
                    var data, response;
                    return _regenerator["default"].wrap(function _callee15$(_context15) {
                      while (1) {
                        switch (_context15.prev = _context15.next) {
                          case 0:
                            data = Buffer.concat([_utils["default"].path_to_buf(path)]);
                            _context15.next = 3;
                            return _send(P1_STAGE_WITNESSES, P2_UNUSED, data);

                          case 3:
                            response = _context15.sent;
                            return _context15.abrupt("return", {
                              path: path,
                              witnessSignatureHex: _utils["default"].buf_to_hex(response)
                            });

                          case 5:
                          case "end":
                            return _context15.stop();
                        }
                      }
                    }, _callee15);
                  }));

                  return function signTx_getWitness(_x17) {
                    return _ref8.apply(this, arguments);
                  };
                }(); //console.log("attest");


                attestedInputs = []; // attest

                _iterator = _createForOfIteratorHelper(inputs);
                _context16.prev = 16;

                _iterator.s();

              case 18:
                if ((_step = _iterator.n()).done) {
                  _context16.next = 26;
                  break;
                }

                _step$value = _step.value, txDataHex = _step$value.txDataHex, outputIndex = _step$value.outputIndex;
                _context16.next = 22;
                return this._attestUtxo(txDataHex, outputIndex);

              case 22:
                _attestation = _context16.sent;
                attestedInputs.push(_attestation);

              case 24:
                _context16.next = 18;
                break;

              case 26:
                _context16.next = 31;
                break;

              case 28:
                _context16.prev = 28;
                _context16.t0 = _context16["catch"](16);

                _iterator.e(_context16.t0);

              case 31:
                _context16.prev = 31;

                _iterator.f();

                return _context16.finish(31);

              case 34:
                _context16.next = 36;
                return signTx_init(attestedInputs.length, outputs.length);

              case 36:
                _i = 0, _attestedInputs = attestedInputs;

              case 37:
                if (!(_i < _attestedInputs.length)) {
                  _context16.next = 44;
                  break;
                }

                attestation = _attestedInputs[_i];
                _context16.next = 41;
                return signTx_addInput(attestation);

              case 41:
                _i++;
                _context16.next = 37;
                break;

              case 44:
                // outputs
                //console.log("outputs");
                _iterator2 = _createForOfIteratorHelper(outputs);
                _context16.prev = 45;

                _iterator2.s();

              case 47:
                if ((_step2 = _iterator2.n()).done) {
                  _context16.next = 62;
                  break;
                }

                output = _step2.value;

                if (!output.address58) {
                  _context16.next = 54;
                  break;
                }

                _context16.next = 52;
                return signTx_addAddressOutput(output.address58, output.amountStr);

              case 52:
                _context16.next = 60;
                break;

              case 54:
                if (!output.path) {
                  _context16.next = 59;
                  break;
                }

                _context16.next = 57;
                return signTx_addChangeOutput(output.path, output.amountStr);

              case 57:
                _context16.next = 60;
                break;

              case 59:
                throw new Error("TODO");

              case 60:
                _context16.next = 47;
                break;

              case 62:
                _context16.next = 67;
                break;

              case 64:
                _context16.prev = 64;
                _context16.t1 = _context16["catch"](45);

                _iterator2.e(_context16.t1);

              case 67:
                _context16.prev = 67;

                _iterator2.f();

                return _context16.finish(67);

              case 70:
                _context16.next = 72;
                return signTx_awaitConfirm();

              case 72:
                _yield$signTx_awaitCo = _context16.sent;
                txHashHex = _yield$signTx_awaitCo.txHashHex;
                //console.log("witnesses");
                witnesses = [];
                _iterator3 = _createForOfIteratorHelper(inputs);
                _context16.prev = 76;

                _iterator3.s();

              case 78:
                if ((_step3 = _iterator3.n()).done) {
                  _context16.next = 86;
                  break;
                }

                input = _step3.value;
                _context16.next = 82;
                return signTx_getWitness(input.path);

              case 82:
                witness = _context16.sent;
                witnesses.push(witness);

              case 84:
                _context16.next = 78;
                break;

              case 86:
                _context16.next = 91;
                break;

              case 88:
                _context16.prev = 88;
                _context16.t2 = _context16["catch"](76);

                _iterator3.e(_context16.t2);

              case 91:
                _context16.prev = 91;

                _iterator3.f();

                return _context16.finish(91);

              case 94:
                return _context16.abrupt("return", {
                  txHashHex: txHashHex,
                  witnesses: witnesses
                });

              case 95:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this, [[16, 28, 31, 34], [45, 64, 67, 70], [76, 88, 91, 94]]);
      }));

      function signTransaction(_x8, _x9) {
        return _signTransaction.apply(this, arguments);
      }

      return signTransaction;
    }()
  }]);
  return Ada;
}();

exports["default"] = Ada;
//# sourceMappingURL=Ada.js.map